from RoboidControl.Thing import Thing
from RoboidControl.Messages.IMessage import IMessage

from typing import Optional, Union
import logging
logger = logging.getLogger(__name__)

class BinaryMsg(IMessage):
    """! A message containing binary data for custom communication
    """
    ## The message ID
    id: int = 0xB1
    ## The length of the message in bytes, excluding the binary data
    #
    ## For the total size of the message self.data_length should be added to this value.
    length: int = 4

    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing] = None):
        """! Create a BinaryMsg
        @param arg1 Either:
        - bytes: contianing the serialized data; or
        - int: containgint the network_id 
        @param thing The thing for which the data is communicated
        """
        if thing is None:
            data = bytes(arg1)
            ## The network ID of the thing
            self.network_id = data[1]
            ## The ID of the thing
            self.thing_id = data[2]
            ## The length of the data
            self.data_length = data[3]
            ## The binary data
            self.data = data[4:]
        else:
            ## The network ID of the thing
            self.network_id = int(arg1)
            ## The ID of the thing
            self.thing_id = thing.id
            ##  The thing for which the binary data is communicated
            self.thing = thing

    def Serialize(self, ref_buffer: list[bytearray]):      
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """ 
        buffer: bytearray = ref_buffer[0]
        ix = self.length
        self.data_length = self.thing.GenerateBinary(buffer, {ix})
        if ix <= self.length:
            return 0
        
        logger.debug(f'Send BinaryMsg [{self.network_id}/{self.thing_id}] {self.data_length}')

        buffer[0] = self.id
        buffer[1] = self.network_id
        buffer[2] = self.thing_id
        buffer[3] = self.data_length
        return ix
