from RoboidControl.Thing import Thing
from RoboidControl.Messages.IMessage import IMessage

from typing import Optional, Union
import logging
logger = logging.getLogger(__name__)

class ModelUrlMsg(IMessage):
    """! Message for communicating the URL for a model of the thing
    """
    ## The message ID
    id = 0x90
    ## The length of the message in bytes, excluding the binary data
    #
    ## For the total size of the message self.url_length should be added to this value.
    length = 4
    
    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing] = None):
        """! Create a ModelUrlMsg
        @param arg1 Either:
        - bytes: containing the serialized data; or
        - int: containing the network_id 
        @param thing The thing for which the data is communicated
        """
        if thing is None:
            data = bytes(arg1)
            ## The network ID of the thing
            self.network_id = data[1]
            ## The ID of the thing
            self.thing_id = data[2]
            ## The url
            self.url = data[ModelUrlMsg.length:]
            ## The length of the encoded url
            self.url_length = len(self.url)
            self.url = self.url.decode("utf-8")
        else:
            ## The network ID of the thing
            self.network_id: int = int(arg1)
            ## The ID of the thing
            self.thing_id: int = thing.id
            if thing.model_url is None:
                ## The URL
                self.url: Optional[str] = None
                ## The length of the encoded URL
                self.url_length = 0
            else:
                ## The URL
                self.url: Optional[str] = thing.model_url.encode('utf-8')
                ## The length of the encoded URL
                self.url_length = len(self.url)
    
    def Serialize(self, buffer_ref: list[bytearray]):
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """ 

        if self.url is None:
            return 0
        
        buffer: bytearray = buffer_ref[0]

        full_length = ModelUrlMsg.length + self.url_length
        if self.url_length == 0 or full_length > len(buffer):
            return 0
        
        logger.debug(f'Send ModelUrlMsg [{self.network_id}/{self.thing_id}] {self.url_length}')

        buffer[0:ModelUrlMsg.length] = [
            ModelUrlMsg.id,
            self.network_id,
            self.thing_id,
            self.url_length
        ]
        # Append the url string
        buffer[ModelUrlMsg.length:full_length] = self.url
        return full_length
    
