from RoboidControl.Messages.IMessage import IMessage

from typing import Union
import logging
logger = logging.getLogger(__name__)

class TextMsg(IMessage):
    """! Message for sending generic text
    """
    ## The message ID
    id: int = 0xB0
    ## The length of the message in bytes without the text itself
    length: int = 2

    def __init__(self, arg1: Union[bytes, str]):
        """! Create a new message for sending
        @param arg1 Either:
        - bytes: containing the serialized data; or
        - str: containing the text
        """
        if isinstance(arg1, bytes):
            data = arg1
            ## The text
            self.text = data[TextMsg.length:]
            ## The length of the encoded text
            self.text_length = len(self.text)
            self.text = self.text.decode("utf-8")
        else:
            text = str(arg1)
            self.text: str = text.encode('utf-8')
            ## The length of the encoded text
            self.text_length = len(self.text)

    def Serialize(self, buffer_ref):
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """ 
        if buffer_ref is None:
            return 0
        
        full_length = TextMsg.length + self.text_length
        if self.text_length == 0 or full_length > len(buffer):
            return 0

        logger.debug(f'Send TextMsg {self.text}')
        
        buffer: bytearray = buffer_ref[0]
        buffer[0:TextMsg.length] = [
            TextMsg.id,
            self.text_length
        ]
        # Append the text string
        buffer[TextMsg.length:full_length] = self.text
        return full_length